@@ -26,7 +26,7 @@ from chamber.shortcuts import get_object_or_none
 from chamber.utils import remove_accent, transaction
 
 from .conf import settings
-from .paginator import BaseOffsetPaginator
+from .paginator import OffsetBasedPaginator
 from .response import (HeadersResponse, RESTCreatedResponse, RESTNoContentResponse, ResponseErrorFactory,
                        ResponseExceptionFactory, ResponseValidationExceptionFactory)
 from .exception import (RESTException, ConflictException, NotAllowedException, DataInvalidException,
@@ -202,13 +202,14 @@ class BaseResource(PermissionsResourceMixin, metaclass=ResourceMetaClass):
     abstract = True
     csrf_exempt = True
     cache = None
-    paginator = BaseOffsetPaginator
+    paginator = OffsetBasedPaginator
     resource_typemapper = {}
     converter_classes = settings.CONVERTERS
     errors_response_class = settings.ERRORS_RESPONSE_CLASS
     error_response_class = settings.ERROR_RESPONSE_CLASS
     field_labels = {}
     requested_fields_manager = DefaultRequestedFieldsManager()
+    renamed_fields = {}
 
     DEFAULT_REST_CONTEXT_MAPPING = {
         'serialization_format': ('HTTP_X_SERIALIZATION_FORMAT', '_serialization_format'),
@@ -219,8 +220,19 @@ class BaseResource(PermissionsResourceMixin, metaclass=ResourceMetaClass):
         'content_type': ('CONTENT_TYPE', '_content_type'),
         'filter': ('HTTP_X_FILTER', 'filter'),
         'order': ('HTTP_X_ORDER', 'order'),
+        'cursor': ('HTTP_X_CURSOR', 'cursor'),
     }
-    renamed_fields = {}
+
+    def update_errors(self, data):
+        if data and self.renamed_fields:
+            data = LazyMappedSerializedData(data, {v: k for k, v in self.renamed_fields.items()}).serialize()
+        return data
+
+    def update_data(self, data):
+        if data and isinstance(data, dict) and self.renamed_fields:
+            return {self.renamed_fields.get(k, k): v for k, v in data.items()}
+        else:
+            return data
 
     def __init__(self, request):
         self.request = request
@@ -284,17 +296,6 @@ class BaseResource(PermissionsResourceMixin, metaclass=ResourceMetaClass):
     def cors_max_age(self):
         return settings.CORS_MAX_AGE
 
-    def update_errors(self, data):
-        if data and self.renamed_fields:
-            data = LazyMappedSerializedData(data, {v: k for k, v in self.renamed_fields.items()}).serialize()
-        return data
-
-    def update_data(self, data):
-        if data and isinstance(data, dict) and self.renamed_fields:
-            return {self.renamed_fields.get(k, k): v for k, v in data.items()}
-        else:
-            return data
-
     def get_dict_data(self):
         return self.update_data(
             self.request.data if hasattr(self.request, 'data') and isinstance(self.request.data, dict) else {}
@@ -379,7 +380,9 @@ class BaseResource(PermissionsResourceMixin, metaclass=ResourceMetaClass):
         if rm in {'POST', 'PUT', 'PATCH'}:
             try:
                 converter = get_converter_from_request(self.request, self.converters, True)
-                self.request.data = self.serializer(self).deserialize(converter.decode(force_text(self.request.body)))
+                self.request.data = self.serializer(self).deserialize(
+                    converter.decode(force_text(self.request.body), resource=self)
+                )
             except (TypeError, ValueError):
                 raise MimerDataException
             except NotImplementedError:
@@ -816,11 +819,14 @@ class BaseObjectResource(DefaultRESTObjectResource, BaseResource):
         pk = self._get_pk()
         if pk:
             return self._get_obj_or_404(pk=pk)
-        qs = self._preload_queryset(self._get_queryset().all())
+        qs = self._preload_queryset(self._get_queryset())
         qs = self._filter_queryset(qs)
         qs = self._order_queryset(qs)
-        paginator = self.paginator(qs, self.request)
-        return HeadersResponse(paginator.page_qs, paginator.headers)
+        if self.paginator:
+            paginator = self.paginator(qs, self.request)
+            return HeadersResponse(paginator.page_qs, paginator.headers)
+        else:
+            return qs
 
     def put(self):
         pk = self._get_pk()
@@ -1063,9 +1069,6 @@ class BaseModelResource(DefaultRESTModelResource, BaseObjectResource):
     def _get_form_fields(self, obj=None):
         return self.form_fields
 
-    def _obj_name(self, obj):
-        return str(obj)
-
     def _generate_form_class(self, inst, exclude=None):
         exclude = [] if exclude is None else exclude
         exclude = list(self._get_exclude(inst)) + exclude
